library flutter_cmislansocket_package;

import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'SocketData.dart';

/// A Calculator.
class CMisLanSocket {
  Socket _socket;
  bool _bConnect = false;
  int _iSeq = 0;
  String _strICIP = "";
  int _iICPort = 0;

  Future<int> connectNS(String strIP, int iPort) async {
    if (_bConnect == true) disConnect();

    //await Socket.connect("10.1.45.127", /*999*/ 760,
    await Socket.connect(strIP, iPort /*760*/,
            timeout: Duration(milliseconds: 100))
        .then((Socket socket) {
      if (socket != null && false == _bConnect) {
        _socket = socket;
        _bConnect = true;
        _sendLoginNSMessage();
      }
      _socket.listen(_getStringData);
    }).catchError((e) {
      print('connectException:$e');
      connectNS(strIP, iPort);
    });
    return 0;
  }

  void connectIC() async {
    if (_bConnect == true) disConnect();

    await Socket.connect(_strICIP, _iICPort,
            timeout: Duration(milliseconds: 100))
        .then((Socket socket) {
      if (socket != null && false == _bConnect) {
        _socket = socket;
        _bConnect = true;
        _sendLoginICMessage();
      }
      _socket.listen(_getStringData);
    }).catchError((e) {
      print('connectException:$e');
      connectIC();
    });
  }

  void disConnect() {
    if (_bConnect == true) {
      _bConnect = false;
      _socket.destroy();
      _socket.close();
      _socket = null;
    }
  }

  void _getStringData(List<int> list) {
    if (list != null) {
      if (10 < list.length) {
        if (list[6] == 0xAF &&
            list[7] == 0x39 &&
            list[8] == 0xD3 &&
            list[9] == 0x85)
          _vDecodeSocketICIP(list);
        else if (list[6] == 0xAF &&
            list[7] == 0x39 &&
            list[8] == 0xD0 &&
            list[9] == 0x97)
          vRegisterSymbol();
        else {
          int iFmtId = list[6];
          int iService = list[7];
          int iTxId = list[8];

          if (iFmtId == 0xE0 && iService == 0x10 && iTxId == 0x00) {
            List<int> listData = new List(list.length - 6 - 3 - 3);
            if (listData != null) {
              List.copyRange(listData, 0, list, 9, list.length - 3);
              vDoMemoryBroadcast(listData);
            }
          }
        }
      }
    }
  }

  void vDoMemoryBroadcast(List<int> list) {
    int iPos = 0;
    int iDataLen = 0;
    String strSymbol = "";

    for (int i = iPos; i < list.length; i++) {
      if (list[i] == 0x00) {
        iDataLen = i;
        break;
      }
    }

    strSymbol = String.fromCharCodes(list, iPos, iDataLen);
    iPos = iDataLen + 1;
    if (strSymbol.isEmpty == false && iPos <= list.length) {
      int iX1DataLen = list.length - iPos;
      List<int> listX1Data = new List(iX1DataLen);

      if (strSymbol.compareTo("2330") == 0) {}
    }

    print(list);
  }

  void _vDecodeSocketICIP(List<int> list) {
    int iLength = list[10];
    List<int> listIP = new List(iLength);
    int iPos = 0;
    if (listIP != null) {
      for (int i = 11; i < list.length - 2; i++) {
        listIP[iPos] = list[i];
        iPos++;
      }

      String strIP = utf8.decode(listIP);
      if (strIP.length > 0) {
        _strICIP = strIP.substring(0, strIP.indexOf(':'));
        _iICPort =
            int.parse(strIP.substring(strIP.indexOf(':') + 1, strIP.length));

        connectIC();
      }
    }
  }

  void vRegisterSymbol() {
    if (_bConnect == true) {
      int iPos = 0;
      Uint8List buf = new Uint8List(256);
      buf[iPos] = 0xAF;
      buf[iPos + 1] = 0x39;
      buf[iPos + 2] = 0xD0;
      buf[iPos + 3] = 0x18;
      iPos += 4;

      String strData = "FRD02945898";
      for (int i = 0; i < strData.length; i++) {
        buf[iPos + i] = strData.codeUnitAt(i);
      }
      iPos += USERID_LEN;

      buf[iPos] = _iSeq % 256;
      iPos++;
      buf[iPos] = _iSeq ~/ 256;
      iPos++;
      buf[iPos] = 0x00;
      iPos++;
      buf[iPos] = 0x06;
      iPos++;
      buf[iPos] = 0x01;
      iPos++;

      buf[iPos] = 0x10;
      iPos++;
      buf[iPos] = 1;
      iPos++;
      buf[iPos] = 0x00;
      iPos++;

      String strSymbol = "2330";

      for (int i = 0; i < strSymbol.length; i++) {
        buf[iPos] = strSymbol.codeUnitAt(i);
        iPos++;
      }
      buf[iPos] = 0x00;
      iPos++;

      _vSendPacket(buf, iPos);
      _iSeq++;
    }
  }

  Future<void> _sendLoginNSMessage() async {
    int iPos = 0;
    Uint8List buf = new Uint8List(256);
    buf[iPos] = 0xAF;
    buf[iPos + 1] = 0x39;
    buf[iPos + 2] = 0xD3;
    buf[iPos + 3] = 0x05;
    iPos += 4;

    _vSendPacket(buf, iPos);
  }

  Future<void> _sendLoginICMessage() async {
    if (_socket != null) {
      int iPos = 0;
      Uint8List buf = new Uint8List(256);
      buf[iPos] = 0xAF;
      buf[iPos + 1] = 0x39;
      buf[iPos + 2] = 0xD0;
      buf[iPos + 3] = 0x17;
      iPos += 4;

      String strData = "FTS03015279";
      for (int i = 0; i < strData.length; i++) {
        buf[iPos + i] = strData.codeUnitAt(i);
      }
      iPos += USERID_LEN;

      strData = '7711092830';
      Uint8List password = new Uint8List(USERID_LEN);

      for (int i = 0; i < password.length; i++) {
        if (i < strData.length) password[i] = strData.codeUnitAt(i);
      }
      decode(password, USERID_LEN, 0x35);

      for (int i = 0; i < password.length; i++) {
        buf[iPos + i] = password[i];
      }
      iPos += USERID_LEN;

      strData = 'FMIDST';
      for (int i = 0; i < strData.length; i++) {
        buf[iPos + i] = strData.codeUnitAt(i);
      }
      iPos += PRODUCTCODE_LEN;

      strData = 'SYSTEX-F1';
      for (int i = 0; i < strData.length; i++) {
        buf[iPos + i] = strData.codeUnitAt(i);
      }
      iPos += PACKAGECODE_LEN;

      iPos += VERSION_LEN;

      buf[iPos] = 0x06;
      buf[iPos + 1] = 0x00;
      buf[iPos + 2] = 0x03;
      iPos += 3;

      buf[iPos] = 0;
      iPos++;

      _vSendPacket(buf, iPos);
    }
  }

  void _vSendPacket(Uint8List uint8list, int iLen) {
    Uint8List message = Uint8List(iLen + 3);
    message[0] = 0x20;
    message[1] = iLen % 256;
    message[2] = iLen ~/ 256;
    iLen += 3;

    for (int i = 0; i < iLen - 3; i++) {
      message[i + 3] = uint8list[i];
    }

    _vSendSockData(message, iLen);
  }

  void _vSendSockData(Uint8List uint8list, int iLen) {
    Uint8List message = Uint8List(iLen + 5);
    var byteData = ByteData.view(message.buffer);
    int iPos = 0;

    message[iPos] = 0x02;
    iPos++;
    message[iPos] = iLen % 256;
    iPos++;
    message[iPos] = iLen ~/ 256;
    iPos++;

    for (int i = 0; i < uint8list.length; i++) {
      message[iPos] = uint8list[i];
      iPos++;
    }

    int iBBC = iBBCDecode(uint8list, iLen, false);
    message[iPos] = iBBC;
    iPos++;

    message[iPos] = 0x03;
    iPos++;

    for (int i = 0; i < message.length; i++) {
      byteData.setUint8(i, message[i]);
    }
    _socket.add(message);
  }

  /// Returns [value] plus 1.
  // int addOne(int value) => value + 1;
}
